@@ -395,9 +395,11 @@ static BonoboUIVerb editor_verbs [] = {
 	BONOBO_UI_UNSAFE_VERB ("InsertSmiley9", smiley_cb),
 	BONOBO_UI_UNSAFE_VERB ("InsertSmiley10", smiley_cb),
 	BONOBO_UI_UNSAFE_VERB ("InsertSmiley11", smiley_cb),
+	BONOBO_UI_UNSAFE_VERB ("InsertSmiley12", smiley_cb),
 
 	BONOBO_UI_UNSAFE_VERB ("IndentMore", command_cb),
 	BONOBO_UI_UNSAFE_VERB ("IndentLess", command_cb),
+	BONOBO_UI_UNSAFE_VERB ("WrapLines", command_cb),
 
 	BONOBO_UI_UNSAFE_VERB ("FormatText", format_text_cb),
 	BONOBO_UI_UNSAFE_VERB ("FormatParagraph", format_paragraph_cb),
@@ -458,6 +460,7 @@ static struct {
 	{"strikeout-toggle", "FormatStrikeout"},
 	{"indent-more", "IndentMore"},
 	{"indent-less", "IndentLess"},
+	{"indent-paragraph", "WrapLines"},
 	{"align-left", "AlignLeft"},
 	{"align-right", "AlignRight"},
 	{"align-center", "AlignCenter"},
@@ -476,7 +479,7 @@ static struct {
 	{"style-header6", "HeadingH6"},
 	{"style-address", "HeadingAddress"},
 	{"style-pre", "HeadingPreformat"},
-	{"style-dot", "HeadingBulletedList"},
+	{"style-itemdot", "HeadingBulletedList"},
 	{"style-itemroman", "HeadingRomanList"},
 	{"style-itemdigit", "HeadingNumberedList"},
 	{"style-itemalpha", "HeadingAlphabeticalList"},
@@ -585,13 +588,6 @@ menubar_update_format (GtkHTMLControlData *cd)
 					      "sensitive", sensitive, &ev);
 		bonobo_ui_component_set_prop (uic, "/commands/FormatPlain",
 					      "sensitive", sensitive, &ev);
-	
-		bonobo_ui_component_set_prop (uic, "/commands/AlignLeft",
-					      "sensitive", sensitive, &ev);		
-		bonobo_ui_component_set_prop (uic, "/commands/AlignRight",
-					      "sensitive", sensitive, &ev);	
-		bonobo_ui_component_set_prop (uic, "/commands/AlignCenter",
-					      "sensitive", sensitive, &ev);	
 
 		bonobo_ui_component_set_prop (uic, "/commands/HeadingH1",
 					      "sensitive", sensitive, &ev);	
@@ -615,7 +611,7 @@ menubar_update_format (GtkHTMLControlData *cd)
 }
 
 void
-menubar_set_languages (GtkHTMLControlData *cd, const gchar *lstr)
+menubar_set_languages (GtkHTMLControlData *cd)
 {
 	GString *str;
 	gboolean enabled;
@@ -627,14 +623,14 @@ menubar_set_languages (GtkHTMLControlData *cd, const gchar *lstr)
 	str = g_string_new (NULL);
 	cd->block_language_changes = TRUE;
 	for (i = 0; i < cd->languages->_length; i ++) {
-		enabled = strstr (lstr, cd->languages->_buffer [i].abbreviation) != NULL;
+		enabled = cd->language && strstr (cd->language, cd->languages->_buffer [i].abbreviation) != NULL;
 		g_string_printf (str, "/commands/SpellLanguage%d", i + 1);
 		bonobo_ui_component_set_prop (cd->uic, str->str, "state", enabled ? "1" : "0", NULL);
 	}
 	cd->block_language_changes = FALSE;
 }
 
-#define SMILEYS 11
+#define SMILEYS 12
 static gchar *smiley [SMILEYS] = {
 	":D",
 	":O",
@@ -646,7 +642,8 @@ static gchar *smiley [SMILEYS] = {
 	":-|",
 	":-/",
 	":-P",
-	":~("
+	":~(",
+	":-Q"
 };
 
 static void
@@ -666,18 +663,22 @@ smiley_cb (BonoboUIComponent *uic, GtkHTMLControlData *cd, const char *cname)
 	}
 }
 
+static void
+menubar_paragraph_style_changed_cb (GtkHTML *html, GtkHTMLParagraphStyle style, GtkHTMLControlData *cd)
+{
+	bonobo_ui_component_set_prop (bonobo_control_get_ui_component (cd->control), "/commands/WrapLines",
+				      "sensitive", style == GTK_HTML_PARAGRAPH_STYLE_PRE ? "1" : "0", NULL);
+}
+
 void
 menubar_setup (BonoboUIComponent  *uic,
 	       GtkHTMLControlData *cd)
 {
 	gchar *domain;
-
 	g_return_if_fail (cd->html != NULL);
 	g_return_if_fail (GTK_IS_HTML (cd->html));
 	g_return_if_fail (BONOBO_IS_UI_COMPONENT (uic));
 
-	/* printf ("xml: %s/%s\n", GTKHTML_DATADIR, "GNOME_GtkHTML_Editor.xml"); */
-
 	/*
 	  FIXME
 
@@ -689,11 +690,33 @@ menubar_setup (BonoboUIComponent  *uic,
 	domain = g_strdup (textdomain (NULL));
 	textdomain (GNOME_EXPLICIT_TRANSLATION_DOMAIN);
 	bonobo_ui_component_add_verb_list_with_data (uic, editor_verbs, cd);
-	bonobo_ui_util_set_ui (uic, GTKHTML_DATADIR, "GNOME_GtkHTML_Editor.xml", "GNOME_GtkHTML_Editor", NULL);
+
+	if (GTK_HTML_CLASS(G_OBJECT_GET_CLASS (cd->html))->use_emacs_bindings) {
+		bonobo_ui_util_set_ui (uic, GTKHTML_DATADIR, "GNOME_GtkHTML_Editor-emacs.xml", "GNOME_GtkHTML_Editor", NULL);
+	} else {
+		bonobo_ui_util_set_ui (uic, GTKHTML_DATADIR, "GNOME_GtkHTML_Editor.xml", "GNOME_GtkHTML_Editor", NULL);
+	}
 
 	spell_create_language_menu (cd);
+	menubar_set_languages (cd);
 	menubar_update_format (cd);
 
 	textdomain (domain);
 	g_free (domain);
+
+	menubar_paragraph_style_changed_cb (cd->html, gtk_html_get_paragraph_style (cd->html), cd);
+
+	if (!cd->has_spell_control_set) {
+		cd->has_spell_control = spell_has_control ();
+		cd->has_spell_control_set = TRUE;
+	}
+
+	if (!cd->has_spell_control) {
+		cd->has_spell_control = FALSE;
+		bonobo_ui_component_set_prop (uic, "/commands/EditSpellCheck", "sensitive", "0", NULL);
+	} else {
+		cd->has_spell_control = TRUE;
+		bonobo_ui_component_set_prop (uic, "/commands/EditSpellCheck", "sensitive", "1", NULL);
+	}
 }
+
