@@ -65,6 +65,7 @@ struct _HTMLEngine {
 	GdkGC *invert_gc;
 
 	gboolean editable;
+	gboolean caret_mode;
 
 	HTMLObject *clipboard;
 	guint       clipboard_len;
@@ -85,7 +86,7 @@ struct _HTMLEngine {
 	gboolean parsing;
 	HTMLTokenizer *ht;
 	HTMLStringTokenizer *st;
-	HTMLObject *clue;
+	HTMLObject *clue;         /* the document root */
 	
 	HTMLObject *flow;
 
@@ -101,9 +102,6 @@ struct _HTMLEngine {
 	gint width;
 	gint height;
 
-	HTMLHAlignType divAlign;
-	HTMLHAlignType pAlign;
-
 	/* Number of tokens parsed in the current time-slice */
 	gint parseCount;
 	gint granularity;
@@ -124,12 +122,11 @@ struct _HTMLEngine {
 	HTMLStack *clueflow_style_stack; /* Clueflow style stack, elements are HTMLClueFlowStyles.  */
 	HTMLStack *frame_stack;
 	HTMLStack *body_stack;
+	HTMLStack *table_stack;
 
 	gchar *url;
 	gchar *target;
 
-	HTMLBlockStackElement *blockStack;
-
 	/* timer id to schedule paint events */
 	guint updateTimer;
 
@@ -179,13 +176,6 @@ struct _HTMLEngine {
            their own.  */
 	gboolean avoid_para;
 
-	/* This is TRUE if we want a paragraph break to be inserted before the
-           next element.  */
-	gboolean pending_para;
-
-	/* Alignment for the pending paragraph we are going to insert.  */
-	HTMLHAlignType pending_para_alignment;
-
 	/* Whether we have the keyboard focus.  */
 	guint have_focus : 1;
 
@@ -262,15 +252,20 @@ struct _HTMLEngine {
 	GList *cut_and_paste_stack;
 
 	gboolean block;
+	gboolean block_images;
 	gint opened_streams;
+	gboolean stopped;
 
 	HTMLObject *focus_object;
+	gint focus_object_offset;
 
 	gboolean save_data;
 	gint saved_step_count;
 
 	gboolean expose;
 	gboolean need_update;
+
+	HTMLObject *parser_clue;  /* the root of the currently parsed block */
 };
 
 /* must be forward referenced *sigh* */
@@ -292,7 +287,7 @@ struct _HTMLEngineClass {
 
 
 /* Object construction.  */
-guint       html_engine_get_type      (void);
+GType       html_engine_get_type      (void);
 HTMLEngine *html_engine_new           (GtkWidget *);
 void        html_engine_realize       (HTMLEngine *engine,
 				       GdkWindow  *window);
@@ -319,6 +314,8 @@ GtkHTMLStream *html_engine_begin            (HTMLEngine  *p,
 					     char        *content_type);
 void           html_engine_parse            (HTMLEngine  *p);
 void           html_engine_stop_parser      (HTMLEngine  *e);
+void           html_engine_stop             (HTMLEngine  *e);
+void           html_engine_flush            (HTMLEngine  *e);
 
 /* Rendering control.  */
 gint  html_engine_calc_min_width       (HTMLEngine *e);
@@ -468,10 +465,11 @@ void html_engine_redraw_selection (HTMLEngine *e);
 
 gboolean    html_engine_focus              (HTMLEngine       *e,
 					    GtkDirectionType  dir);
-HTMLObject *html_engine_get_focus_object   (HTMLEngine       *e);
+HTMLObject *html_engine_get_focus_object   (HTMLEngine       *e,
+					    gint             *offset);
 void        html_engine_set_focus_object   (HTMLEngine       *e,
-					    HTMLObject       *o);
-void        html_engine_draw_focus_object  (HTMLEngine       *e);
+					    HTMLObject       *o,
+					    gint              offset);
 
 HTMLMap *html_engine_get_map  (HTMLEngine  *e,
 			       const gchar *name);
@@ -480,4 +478,9 @@ gboolean html_engine_selection_contains_object_type (HTMLEngine *e,
 						     HTMLType obj_type);
 gboolean html_engine_selection_contains_link        (HTMLEngine *e);
 
+gint  html_engine_get_left_border    (HTMLEngine *e);
+gint  html_engine_get_right_border   (HTMLEngine *e);
+gint  html_engine_get_top_border     (HTMLEngine *e);
+gint  html_engine_get_bottom_border  (HTMLEngine *e);
+
 #endif /* _HTMLENGINE_H_ */
