@@ -24,6 +24,7 @@
 #include <gtk/gtk.h>
 #include <string.h>
 #include "gtkhtml.h"
+#include "gtkhtml-stream.h"
 #include "htmltokenizer.h"
 #include "gtkhtml-private.h"
 #include "htmlcolorset.h"
@@ -35,10 +36,6 @@
 #include "htmlselection.h"
 #include "htmlsettings.h"
 
-#ifndef USE_SCROLLED_WINDOW
-#include <gal/widgets/e-scroll-frame.h>
-#endif
-
 HTMLFrameClass html_frame_class;
 static HTMLEmbeddedClass *parent_class = NULL;
 static gboolean calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs);
@@ -48,86 +45,21 @@ frame_url_requested (GtkHTML *html, const char *url, GtkHTMLStream *handle, gpoi
 {
 	HTMLFrame *frame = HTML_FRAME (data);
 	GtkHTML *parent = GTK_HTML (HTML_EMBEDDED(frame)->parent);
+
+	if (!html->engine->stopped)
+		g_signal_emit_by_name (parent->engine, "url_requested", url, handle);
+}
+
+static void
+frame_set_base (GtkHTML *html, const gchar *url, gpointer data)
+{
 	char *new_url = NULL;
-	
-	/* FIXME this is not exactly the single safest method of expanding a relative url */
-	if (frame->url && !strstr (url, ":")) {
-		char *base = g_strdup (frame->url);
-		if (*url == '/' && strstr (base, ":")) {
-			int i = 0;
-			char *cur = base; 
-			
-			while (*cur != '\0') {
-				if (*cur == '/') i++;
-				
-				if (i == 3) {
-					*(++cur) = '\0';
-					break;
-				}
-				cur++;
-			}
-			new_url = g_strconcat (base, "/", url, NULL);
-		} else if (*url == '/' && (*base == '/' || *base == '\0')) {
-			new_url = NULL;
-		} else {
-			new_url = g_strconcat (base, "/", url, NULL);
-		}		
-		g_free (base);
-	}
-	g_warning ("url = %s, frame->url = %s, new_url = %s", url, frame->url, new_url);
-	
-	gtk_signal_emit_by_name (GTK_OBJECT (parent->engine), "url_requested", new_url ? new_url : url,
-				 handle);
-	
+
+	new_url = gtk_html_get_url_base_relative (html, url);
+	gtk_html_set_base (html, new_url);
 	g_free (new_url);
 }
 
-static void
-frame_set_base (GtkHTML *html, const gchar *base, gpointer data)
-{
-	HTMLFrame *frame = HTML_FRAME (data);
-	char *new_url = NULL;
-	char *cur;
-	
-	/* FIXME this is not exactly the single safest method of expanding a relative url */
-	if (frame->url && !strstr (base, ":"))
-		new_url = g_strconcat (frame->url, "/", base, NULL);
-	else 
-		new_url = g_strdup (base);
-
-	cur = new_url + strlen (new_url);
-	while (cur >= new_url) {
-		if (*cur == '/') {
-			break;
-		}
-		cur--;
-	}	
-	*(cur + 1)= '\0';
-			
-	g_warning ("base = %s, frame->url = %s, new_url = %s", base, frame->url, new_url);
-
-	g_free (frame->url);
-	frame->url = new_url;
-}
-
-static void
-frame_on_url (GtkHTML *html, const gchar *url, gpointer data)
-{
-	HTMLFrame *frame = HTML_FRAME (data);
-	GtkHTML *parent = GTK_HTML (HTML_EMBEDDED(frame)->parent);
-
-	gtk_signal_emit_by_name (GTK_OBJECT (parent), "on_url", url);
-}
-
-static void
-frame_link_clicked (GtkHTML *html, const gchar *url, gpointer data)
-{
-	HTMLFrame *frame = HTML_FRAME (data);
-	GtkHTML *parent = GTK_HTML (HTML_EMBEDDED(frame)->parent);
-
-	gtk_signal_emit_by_name (GTK_OBJECT (parent), "link_clicked", url);
-}
-
 static void
 frame_submit (GtkHTML *html, 
 	      const gchar *method, 
@@ -138,7 +70,7 @@ frame_submit (GtkHTML *html,
 	HTMLFrame *frame = HTML_FRAME (data);
 	GtkHTML *parent = GTK_HTML (HTML_EMBEDDED(frame)->parent);
 
-	gtk_signal_emit_by_name (GTK_OBJECT (parent), "submit", method, action, encoding);
+	g_signal_emit_by_name (parent, "submit", method, action, encoding);
 }
 
 static void
@@ -158,7 +90,7 @@ frame_object_requested (GtkHTML *html, GtkHTMLEmbedded *eb, gpointer data)
 	gboolean ret_val;
 
 	ret_val = FALSE;
-	gtk_signal_emit_by_name (GTK_OBJECT (parent), "object_requested", eb, &ret_val);
+	g_signal_emit_by_name (parent, "object_requested", eb, &ret_val);
 	return ret_val;
 }
 
@@ -166,10 +98,10 @@ static void
 frame_set_gdk_painter (HTMLFrame *frame, HTMLPainter *painter)
 {
 	if (painter)
-		gtk_object_ref (GTK_OBJECT (painter));
+		g_object_ref (G_OBJECT (painter));
 	
 	if (frame->gdk_painter)
-		gtk_object_unref (GTK_OBJECT (frame->gdk_painter));
+		g_object_unref (G_OBJECT (frame->gdk_painter));
 
 	frame->gdk_painter = painter;
 }
@@ -223,7 +155,7 @@ set_max_width (HTMLObject *o, HTMLPainter *painter, gint max_width)
 	HTMLEngine *e = GTK_HTML (HTML_FRAME (o)->html)->engine;
 
 	o->max_width = max_width;
-	html_object_set_max_width (e->clue, e->painter, max_width - e->leftBorder - e->rightBorder);
+	html_object_set_max_width (e->clue, e->painter, max_width - (html_engine_get_left_border (e) + html_engine_get_right_border (e)));
 }
 
 /* static void
@@ -256,19 +188,19 @@ draw (HTMLObject *o,
 {
 	HTMLFrame   *frame  = HTML_FRAME (o);
 	HTMLEngine   *e       = GTK_HTML (frame->html)->engine;
-	ArtIRect paint;
+	GdkRectangle paint;
 
 	if (GTK_OBJECT_TYPE (e->painter) == HTML_TYPE_PRINTER) {
 		gint pixel_size = html_painter_get_pixel_size (e->painter);
-		html_object_calc_intersection (o, &paint, x, y, width, height);
-		if (art_irect_empty (&paint))
+
+		if (!html_object_intersect (o, &paint, x, y, width, height))
 			return;
 
 		html_object_draw (e->clue, e->painter,
 				  x, y,
-				  width - pixel_size * (e->leftBorder + e->rightBorder),
-				  height - pixel_size * (e->topBorder + e->bottomBorder),
-				  tx + pixel_size * e->leftBorder, ty + pixel_size * e->topBorder);
+				  width - pixel_size * (html_engine_get_left_border (e) + html_engine_get_right_border (e)),
+				  height - pixel_size * (html_engine_get_top_border (e) + html_engine_get_bottom_border (e)),
+				  tx + pixel_size * html_engine_get_left_border (e), ty + pixel_size * html_engine_get_top_border (e));
 	} else
 		(*HTML_OBJECT_CLASS (parent_class)->draw) (o, p, x, y, width, height, tx, ty);
 }
@@ -279,7 +211,7 @@ set_painter (HTMLObject *o, HTMLPainter *painter)
 	HTMLFrame *frame;
 
 	frame = HTML_FRAME (o);
-	if (GTK_OBJECT_TYPE (GTK_HTML (frame->html)->engine->painter) != HTML_TYPE_PRINTER) {
+	if (G_OBJECT_TYPE (GTK_HTML (frame->html)->engine->painter) != HTML_TYPE_PRINTER) {
 		frame_set_gdk_painter (frame, GTK_HTML (frame->html)->engine->painter);
 	}
 	
@@ -328,7 +260,7 @@ calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs)
 		height = html_engine_get_doc_height (e);
 		width = html_engine_get_doc_width (e);
 
-		gtk_widget_set_usize (frame->scroll, width, height);
+		gtk_widget_set_size_request (frame->scroll, width, height);
 		gtk_widget_queue_resize (frame->scroll);
 		
 		html_frame_set_scrolling (frame, GTK_POLICY_NEVER);
@@ -406,8 +338,8 @@ check_point (HTMLObject *self,
 	    || y >= self->y + self->descent || y < self->y - self->ascent)
 		return NULL;
 
-	x -= self->x + e->leftBorder - e->x_offset;
-	y -= self->y - self->ascent + e->topBorder - e->y_offset;
+	x -= self->x + html_engine_get_left_border (e) - e->x_offset;
+	y -= self->y - self->ascent + html_engine_get_top_border (e) - e->y_offset;
 
 	if (for_cursor && (x < 0 || y < e->clue->y - e->clue->ascent))
 		return html_object_check_point (e->clue, e->painter, 0, e->clue->y - e->clue->ascent,
@@ -433,6 +365,17 @@ append_selection_string (HTMLObject *self,
 	html_object_append_selection_string (GTK_HTML (HTML_FRAME (self)->html)->engine->clue, buffer);
 }
 
+static void
+reparent (HTMLEmbedded *emb, GtkWidget *html)
+{
+	HTMLFrame *frame = HTML_FRAME (emb);
+
+	gtk_html_set_iframe_parent (GTK_HTML (frame->html), 
+				    html,
+				    GTK_HTML (frame->html)->frame);
+	(* HTML_EMBEDDED_CLASS (parent_class)->reparent) (emb, html);
+}
+
 static gboolean
 select_range (HTMLObject *self,
 	      HTMLEngine *engine,
@@ -453,7 +396,7 @@ destroy (HTMLObject *o)
 	frame_set_gdk_painter (frame, NULL);
 
 	if (frame->html) {
-		gtk_signal_disconnect_by_data (GTK_OBJECT (frame->html), o);
+		g_signal_handlers_disconnect_matched (frame->html, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, o);
 		frame->html = NULL;
 	}
 	g_free ((frame)->url);
@@ -486,13 +429,9 @@ html_frame_set_margin_height (HTMLFrame *frame, gint margin_height)
 void
 html_frame_set_scrolling (HTMLFrame *frame, GtkPolicyType scroll)
 {
-#if E_USE_SCROLLED_WINDOW
+
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (frame->scroll),
 					scroll, scroll);
-#else
-	e_scroll_frame_set_policy (E_SCROLL_FRAME (frame->scroll),
-				   scroll, scroll);
-#endif					
 }
 
 void
@@ -506,7 +445,7 @@ html_frame_set_size (HTMLFrame *frame, gint width, gint height)
 	if (height > 0)
 		frame->height = height;
 	
-	gtk_widget_set_usize (frame->scroll, width, height);
+	gtk_widget_set_size_request (frame->scroll, width, height);
 }
 
 void 
@@ -523,8 +462,8 @@ html_frame_init (HTMLFrame *frame,
 	GtkWidget *new_widget;
 	GtkHTML   *new_html;
 	GtkHTML   *parent_html;
-	GtkHTMLStream *handle;
 	GtkWidget *scrolled_window;
+	gint depth;
 
 	g_assert (GTK_IS_HTML (parent));
 	parent_html = GTK_HTML (parent);
@@ -532,14 +471,9 @@ html_frame_init (HTMLFrame *frame,
 	html_embedded_init (em, HTML_EMBEDDED_CLASS (klass),
 			    parent, NULL, NULL);
 	
-#if USE_SCROLLED_WINDOW
 	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
-#else
-	scrolled_window = e_scroll_frame_new (NULL, NULL);
-	e_scroll_frame_set_shadow_type (E_SCROLL_FRAME (scrolled_window), 
-					border ? GTK_SHADOW_IN : GTK_SHADOW_NONE);
-
-#endif
+	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_window),
+					     border ? GTK_SHADOW_IN : GTK_SHADOW_NONE);
 
 	new_widget = gtk_html_new ();
 	new_html = GTK_HTML (new_widget);
@@ -547,63 +481,63 @@ html_frame_init (HTMLFrame *frame,
 	new_tokenizer = html_tokenizer_clone (parent_html->engine->ht);
 
 	html_engine_set_tokenizer (new_html->engine, new_tokenizer);
-	gtk_object_unref (GTK_OBJECT (new_tokenizer));
+	g_object_unref (G_OBJECT (new_tokenizer));
 	new_tokenizer = NULL;
 
 	gtk_html_set_default_content_type (new_html,
 					   parent_html->priv->content_type);
 	frame->html = new_widget;
-	gtk_html_set_iframe_parent (new_html, parent, HTML_OBJECT (frame));
+	depth = gtk_html_set_iframe_parent (new_html, parent, HTML_OBJECT (frame));
 	gtk_container_add (GTK_CONTAINER (scrolled_window), new_widget);
 	gtk_widget_show (new_widget);
 
+	frame->url = g_strdup (src);
 	frame->width = width;
 	frame->height = height;
 	frame->gdk_painter = NULL;
-	frame->url = NULL;
+	gtk_html_set_base (new_html, src);
 
-	if (src && strcmp (src, "")) {
-		handle = gtk_html_begin (new_html);
-		gtk_signal_emit_by_name (GTK_OBJECT (new_html->engine), 
-					 "url_requested", src, handle);
+	gtk_html_set_base (new_html, src);
+	g_signal_connect (new_html, "url_requested", G_CALLBACK (frame_url_requested), frame);
 
-		frame_set_base (new_html, src, frame);
-	} else {		
+	if (parent_html->engine->stopped) {
+		gtk_html_stop (new_html);
 		gtk_html_load_empty (new_html);
+	} else {
+		GtkHTMLStream *handle;
+
+		handle = gtk_html_begin (new_html);
+		g_signal_emit_by_name (parent_html->engine, "url_requested", src, handle);
 	}
+
 	new_html->engine->clue->parent = HTML_OBJECT (frame);
 
-
-	gtk_signal_connect (GTK_OBJECT (new_html), "url_requested",
-			    GTK_SIGNAL_FUNC (frame_url_requested),
-			    (gpointer)frame);
+#if 0
+	/* NOTE: because of peculiarities of the frame/gtkhtml relationship
+	 * on_url and link_clicked are emitted from the toplevel widget not
+	 * proxied like url_requested is.
+	 */
 	gtk_signal_connect (GTK_OBJECT (new_html), "on_url",
 			    GTK_SIGNAL_FUNC (frame_on_url), 
 			    (gpointer)frame);
 	gtk_signal_connect (GTK_OBJECT (new_html), "link_clicked",
 			    GTK_SIGNAL_FUNC (frame_link_clicked),
 			    (gpointer)frame);	
-	gtk_signal_connect (GTK_OBJECT (new_html), "size_changed",
-			    GTK_SIGNAL_FUNC (frame_size_changed),
-			    (gpointer)frame);	
-	gtk_signal_connect (GTK_OBJECT (new_html), "object_requested",
-			    GTK_SIGNAL_FUNC (frame_object_requested),
-			    (gpointer)frame);	
-	gtk_signal_connect (GTK_OBJECT (new_html), "submit",
-			    GTK_SIGNAL_FUNC (frame_submit),
-			    (gpointer)frame);
+#endif
+	g_signal_connect (new_html, "size_changed", G_CALLBACK (frame_size_changed), frame);	
+	g_signal_connect (new_html, "object_requested", G_CALLBACK (frame_object_requested), frame);	
+	g_signal_connect (new_html, "submit", G_CALLBACK (frame_submit), frame);
+	g_signal_connect (new_html, "set_base", G_CALLBACK (frame_set_base), frame);
 
 	html_frame_set_margin_height (frame, 0);
 	html_frame_set_margin_width (frame, 0);
-	/*
-	  gtk_signal_connect (GTK_OBJECT (new_html), "set_base",
-	  GTK_SIGNAL_FUNC (frame_set_base), (gpointer)frame);
 
+	/*
 	  gtk_signal_connect (GTK_OBJECT (html), "button_press_event",
 	  GTK_SIGNAL_FUNC (frame_button_press_event), frame);
 	*/
 
-	gtk_widget_set_usize (scrolled_window, width, height);
+	gtk_widget_set_size_request (scrolled_window, width, height);
 
 	gtk_widget_show (scrolled_window);	
 	frame->scroll = scrolled_window;
@@ -611,8 +545,7 @@ html_frame_init (HTMLFrame *frame,
 
 	html_embedded_set_widget (em, scrolled_window);
 
-	gtk_signal_connect(GTK_OBJECT (scrolled_window), "button_press_event",
-			   GTK_SIGNAL_FUNC (html_frame_grab_cursor), NULL);
+	g_signal_connect (scrolled_window, "button_press_event", G_CALLBACK (html_frame_grab_cursor), NULL);
 
 	/* inherit the current colors from our parent */
 	html_colorset_set_unchanged (new_html->engine->defaultSettings->color_set,
@@ -621,14 +554,10 @@ html_frame_init (HTMLFrame *frame,
 				     parent_html->engine->settings->color_set);
 	html_painter_set_focus (new_html->engine->painter, parent_html->engine->have_focus);
 	/*
-	gtk_signal_connect (GTK_OBJECT (html), "title_changed",
-			    GTK_SIGNAL_FUNC (title_changed_cb), (gpointer)app);
-	gtk_signal_connect (GTK_OBJECT (html), "button_press_event",
-			    GTK_SIGNAL_FUNC (on_button_press_event), popup_menu);
-	gtk_signal_connect (GTK_OBJECT (html), "redirect",
-			    GTK_SIGNAL_FUNC (on_redirect), NULL);
-	gtk_signal_connect (GTK_OBJECT (html), "object_requested",
-			    GTK_SIGNAL_FUNC (object_requested_cmd), NULL);
+	g_signal_connect (html, "title_changed", G_CALLBACK (title_changed_cb), app);
+	g_signal_connect (html, "button_press_event", G_CALLBACK (on_button_press_event), popup_menu);
+	g_signal_connect (html, "redirect", G_CALLBACK (on_redirect), NULL);
+	g_signal_connect (html, "object_requested", G_CALLBACK (object_requested_cmd), NULL);
 	*/
 }
 
@@ -669,9 +598,10 @@ html_frame_class_init (HTMLFrameClass *klass,
 	object_class->get_engine              = get_engine;
 	object_class->check_point             = check_point;
 	object_class->is_container            = is_container;
-	object_class->draw_background         = draw_background;
 	object_class->append_selection_string = append_selection_string;
 	object_class->select_range            = select_range;
+
+	embedded_class->reparent = reparent;
 }
 
 
